@@ -30,7 +30,7 @@
 from supybot.test import *
 
 class WebTestCase(ChannelPluginTestCase):
-    plugins = ('Web',)
+    plugins = ('Web', 'Admin',)
     timeout = 10
     if network:
         def testHeaders(self):
@@ -67,7 +67,13 @@ class WebTestCase(ChannelPluginTestCase):
                         'title http://www.youtube.com/watch?v=x4BtiqPN4u8')
             self.assertResponse(
                     'title http://www.thefreedictionary.com/don%27t',
-                    "don't - definition of don't by The Free Dictionary")
+                    "Don't - definition of don't by The Free Dictionary")
+            self.assertRegexp(
+                    'title '
+                    'https://twitter.com/rlbarnes/status/656554266744586240',
+                    '"PSA: In Firefox 44 Nightly, "http:" pages with '
+                    '<input type="password"> are now marked insecure. '
+                    'https://t.co/qS9LxuRPdm"$')
 
         def testTitleSnarfer(self):
             try:
@@ -92,6 +98,34 @@ class WebTestCase(ChannelPluginTestCase):
             finally:
                 conf.supybot.plugins.Web.nonSnarfingRegexp.setValue(snarf)
 
+        def testSnarferIgnore(self):
+            conf.supybot.plugins.Web.titleSnarfer.setValue(True)
+            (oldprefix, self.prefix) = (self.prefix, 'foo!bar@baz')
+            try:
+                self.assertSnarfRegexp('http://google.com/', 'Google')
+                self.assertNotError('admin ignore add %s' % self.prefix)
+                self.assertSnarfNoResponse('http://google.com/')
+                self.assertNoResponse('title http://www.google.com/')
+            finally:
+                conf.supybot.plugins.Web.titleSnarfer.setValue(False)
+                (self.prefix, oldprefix) = (oldprefix, self.prefix)
+                self.assertNotError('admin ignore remove %s' % oldprefix)
+
+        def testSnarferNotIgnore(self):
+            conf.supybot.plugins.Web.titleSnarfer.setValue(True)
+            conf.supybot.plugins.Web.checkIgnored.setValue(False)
+            (oldprefix, self.prefix) = (self.prefix, 'foo!bar@baz')
+            try:
+                self.assertSnarfRegexp('https://google.com/', 'Google')
+                self.assertNotError('admin ignore add %s' % self.prefix)
+                self.assertSnarfRegexp('https://www.google.com/', 'Google')
+                self.assertNoResponse('title http://www.google.com/')
+            finally:
+                conf.supybot.plugins.Web.titleSnarfer.setValue(False)
+                conf.supybot.plugins.Web.checkIgnored.setValue(True)
+                (self.prefix, oldprefix) = (oldprefix, self.prefix)
+                self.assertNotError('admin ignore remove %s' % oldprefix)
+
         def testWhitelist(self):
             fm = conf.supybot.plugins.Web.fetch.maximum()
             uw = conf.supybot.plugins.Web.urlWhitelist()
