@@ -1,6 +1,6 @@
 ###
 # Copyright (c) 2002-2005, Jeremiah Fincher
-# Copyright (c) 2009,2011, James McCoy
+# Copyright (c) 2009,2011,2015 James McCoy
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -41,16 +41,19 @@ from __future__ import print_function
 import re
 import sys
 import time
+import base64
 import random
 import string
 import textwrap
 import functools
-from cStringIO import StringIO as sio
 
 from . import utils
-from . import minisix
+from .utils import minisix
 from .version import version
 
+from .i18n import PluginInternationalization
+_ = PluginInternationalization()
+
 def debug(s, *args):
     """Prints a debug string.  Most likely replaced by our logging debug."""
     print('***', s % args)
@@ -113,8 +116,8 @@ def toLower(s, casemapping=None):
 def strEqual(nick1, nick2):
     """s1, s2 => bool
     Returns True if nick1 == nick2 according to IRC case rules."""
-    assert isinstance(nick1, basestring)
-    assert isinstance(nick2, basestring)
+    assert isinstance(nick1, minisix.string_types)
+    assert isinstance(nick2, minisix.string_types)
     return toLower(nick1) == toLower(nick2)
 
 nickEqual = strEqual
@@ -171,7 +174,7 @@ def _hostmaskPatternEqual(pattern, hostmask):
     except KeyError:
         # We make our own regexps, rather than use fnmatch, because fnmatch's
         # case-insensitivity is not IRC's case-insensitity.
-        fd = sio()
+        fd = minisix.io.StringIO()
         for c in pattern:
             if c == '*':
                 fd.write('.*')
@@ -297,6 +300,10 @@ def bold(s):
     """Returns the string s, bolded."""
     return '\x02%s\x02' % s
 
+def italic(s):
+    """Returns the string s, italicised."""
+    return '\x1D%s\x1D' % s
+
 def reverse(s):
     """Returns the string s, reverse-videoed."""
     return '\x16%s\x16' % s
@@ -353,6 +360,10 @@ def stripBold(s):
     """Returns the string s, with bold removed."""
     return s.replace('\x02', '')
 
+def stripItalic(s):
+    """Returns the string s, with italics removed."""
+    return s.replace('\x1d', '')
+
 _stripColorRe = re.compile(r'\x03(?:\d{1,2},\d{1,2}|\d{1,2}|,\d{1,2}|)')
 def stripColor(s):
     """Returns the string s, with color removed."""
@@ -373,8 +384,126 @@ def stripFormatting(s):
     s = stripBold(s)
     s = stripReverse(s)
     s = stripUnderline(s)
+    s = stripItalic(s)
     return s.replace('\x0f', '').replace('\x0F', '')
 
+_containsFormattingRe = re.compile(r'[\x02\x03\x16\x1f]')
+def formatWhois(irc, replies, caller='', channel='', command='whois'):
+    """Returns a string describing the target of a WHOIS command.
+
+    Arguments are:
+    * irc: the irclib.Irc object on which the replies was received
+
+    * replies: a dict mapping the reply codes ('311', '312', etc.) to their
+      corresponding ircmsg.IrcMsg
+
+    * caller: an optional nick specifying who requested the whois information
+
+    * channel: an optional channel specifying where the reply will be sent
+
+    If provided, caller and channel will be used to avoid leaking information
+    that the caller/channel shouldn't be privy to.
+    """
+    hostmask = '@'.join(replies['311'].args[2:4])
+    nick = replies['318'].args[1]
+    user = replies['311'].args[-1]
+    START_CODE = '311' if command == 'whois' else '314'
+    hostmask = '@'.join(replies[START_CODE].args[2:4])
+    user = replies[START_CODE].args[-1]
+    if _containsFormattingRe.search(user) and user[-1] != '\x0f':
+        # For good measure, disable any formatting
+        user = '%s\x0f' % user
+    if '319' in replies:
+        channels = []
+        for msg in replies['319']:
+            channels.extend(msg.args[-1].split())
+        ops = []
+        voices = []
+        normal = []
+        halfops = []
+        for chan in channels:
+            origchan = chan
+            chan = chan.lstrip('@%+~!')
+            # UnrealIRCd uses & for user modes and disallows it as a
+            # channel-prefix, flying in the face of the RFC.  Have to
+            # handle this specially when processing WHOIS response.
+            testchan = chan.lstrip('&')
+            if testchan != chan and irc.isChannel(testchan):
+                chan = testchan
+            diff = len(chan) - len(origchan)
+            modes = origchan[:diff]
+            chanState = irc.state.channels.get(chan)
+            # The user is in a channel the bot is in, so the ircd may have
+            # responded with otherwise private data.
+            if chanState:
+                # Skip channels the caller isn't in.  This prevents
+                # us from leaking information when the channel is +s or the
+                # target is +i.
+                if caller not in chanState.users:
+                    continue
+                # Skip +s/+p channels the target is in only if the reply isn't
+                # being sent to that channel.
+                if set(('p', 's')) & set(chanState.modes.keys()) and \
+                   not strEqual(channel or '', chan):
+                    continue
+            if not modes:
+                normal.append(chan)
+            elif utils.iter.any(lambda c: c in modes,('@', '&', '~', '!')):
+                ops.append(chan)
+            elif utils.iter.any(lambda c: c in modes, ('%',)):
+                halfops.append(chan)
+            elif utils.iter.any(lambda c: c in modes, ('+',)):
+                voices.append(chan)
+        L = []
+        if ops:
+            L.append(format(_('is an op on %L'), ops))
+        if halfops:
+            L.append(format(_('is a halfop on %L'), halfops))
+        if voices:
+            L.append(format(_('is voiced on %L'), voices))
+        if normal:
+            if L:
+                L.append(format(_('is also on %L'), normal))
+            else:
+                L.append(format(_('is on %L'), normal))
+    else:
+        if command == 'whois':
+            L = [_('isn\'t on any publicly visible channels')]
+        else:
+            L = []
+    channels = format('%L', L)
+    if '317' in replies:
+        idle = utils.timeElapsed(replies['317'].args[2])
+        signon = utils.str.timestamp(float(replies['317'].args[3]))
+    else:
+        idle = '<unknown>'
+        signon = '<unknown>'
+    if '312' in replies:
+        server = replies['312'].args[2]
+        if len(replies['312']) > 3:
+            signoff = replies['312'].args[3]
+    else:
+        server = '<unknown>'
+    if '301' in replies:
+        away = ' %s is away: %s.' % (nick, replies['301'].args[2])
+    else:
+        away = ''
+    if '320' in replies:
+        if replies['320'].args[2]:
+            identify = ' identified'
+        else:
+            identify = ''
+    else:
+        identify = ''
+    if command == 'whois':
+        s = _('%s (%s) has been%s on server %s since %s (idle for %s). %s '
+              '%s.%s') % (user, hostmask, identify, server,
+                          signon, idle, nick, channels, away)
+    else:
+        s = _('%s (%s) has been%s on server %s and disconnected on %s.') % \
+                (user, hostmask, identify, server, signoff)
+    return s
+
 class FormatContext(object):
     def __init__(self):
         self.reset()
@@ -408,7 +537,7 @@ class FormatContext(object):
 
 class FormatParser(object):
     def __init__(self, s):
-        self.fd = sio(s)
+        self.fd = minisix.io.StringIO(s)
         self.last = None
 
     def getChar(self):
@@ -488,10 +617,10 @@ def isValidArgument(s):
 
 def safeArgument(s):
     """If s is unsafe for IRC, returns a safe version."""
-    if sys.version_info[0] < 3 and isinstance(s, unicode):
+    if minisix.PY2 and isinstance(s, unicode):
         s = s.encode('utf-8')
-    elif (sys.version_info[0] < 3 and not isinstance(s, basestring)) or \
-            (sys.version_info[0] >= 3 and not isinstance(s, str)):
+    elif (minisix.PY2 and not isinstance(s, minisix.string_types)) or \
+            (minisix.PY3 and not isinstance(s, str)):
         debug('Got a non-string in safeArgument: %r', s)
         s = str(s)
     if isValidArgument(s):
@@ -519,7 +648,7 @@ def dccIP(ip):
 
 def unDccIP(i):
     """Takes an integer DCC IP and return a normal string IP."""
-    assert isinstance(i, (int, long)), '%r is not an number.' % i
+    assert isinstance(i, minisix.integer_types), '%r is not an number.' % i
     L = []
     while len(L) < 4:
         L.append(i % 256)
@@ -659,7 +788,7 @@ mircColors = IrcDict({
 })
 
 # We'll map integers to their string form so mircColor is simpler.
-for (k, v) in mircColors.items():
+for (k, v) in list(mircColors.items()):
     if k is not None: # Ignore empty string for None.
         sv = str(v)
         mircColors[sv] = sv
@@ -690,11 +819,12 @@ def standardSubstitute(irc, msg, text, env=None):
         'm': localtime[4], 'min': localtime[4], 'minute': localtime[4],
         's': localtime[5], 'sec': localtime[5], 'second': localtime[5],
         'tz': time.strftime('%Z', localtime),
-        'version': 'Supybot %s' % version,
+        'version': 'Limnoria %s' % version,
         })
     if irc:
         vars.update({
             'botnick': irc.nick,
+            'network': irc.network,
             })
 
     if msg:
@@ -740,6 +870,238 @@ def standardSubstitute(irc, msg, text, env=None):
     t.idpattern = '[a-zA-Z][a-zA-Z0-9]*'
     return t.safe_substitute(vars)
 
+
+
+AUTHENTICATE_CHUNK_SIZE = 400
+def authenticate_generator(authstring, base64ify=True):
+    if base64ify:
+        authstring = base64.b64encode(authstring)
+        if minisix.PY3:
+            authstring = authstring.decode()
+    # +1 so we get an empty string at the end if len(authstring) is a multiple
+    # of AUTHENTICATE_CHUNK_SIZE (including 0)
+    for n in range(0, len(authstring)+1, AUTHENTICATE_CHUNK_SIZE):
+        chunk = authstring[n:n+AUTHENTICATE_CHUNK_SIZE] or '+'
+        yield chunk
+
+class AuthenticateDecoder(object):
+    def __init__(self):
+        self.chunks = []
+        self.ready = False
+    def feed(self, msg):
+        assert msg.command == 'AUTHENTICATE'
+        chunk = msg.args[0]
+        if chunk == '+' or len(chunk) != AUTHENTICATE_CHUNK_SIZE:
+            self.ready = True
+        if chunk != '+':
+            if minisix.PY3:
+                chunk = chunk.encode()
+            self.chunks.append(chunk)
+    def get(self):
+        assert self.ready
+        return base64.b64decode(b''.join(self.chunks))
+
+
+numerics = {
+    # <= 2.10
+        # Reply
+        '001': 'RPL_WELCOME',
+        '002': 'RPL_YOURHOST',
+        '003': 'RPL_CREATED',
+        '004': 'RPL_MYINFO',
+        '005': 'RPL_BOUNCE',
+        '302': 'RPL_USERHOST',
+        '303': 'RPL_ISON',
+        '301': 'RPL_AWAY',
+        '305': 'RPL_UNAWAY',
+        '306': 'RPL_NOWAWAY',
+        '311': 'RPL_WHOISUSER',
+        '312': 'RPL_WHOISSERVER',
+        '313': 'RPL_WHOISOPERATOR',
+        '317': 'RPL_WHOISIDLE',
+        '318': 'RPL_ENDOFWHOIS',
+        '319': 'RPL_WHOISCHANNELS',
+        '314': 'RPL_WHOWASUSER',
+        '369': 'RPL_ENDOFWHOWAS',
+        '321': 'RPL_LISTSTART',
+        '322': 'RPL_LIST',
+        '323': 'RPL_LISTEND',
+        '325': 'RPL_UNIQOPIS',
+        '324': 'RPL_CHANNELMODEIS',
+        '331': 'RPL_NOTOPIC',
+        '332': 'RPL_TOPIC',
+        '341': 'RPL_INVITING',
+        '342': 'RPL_SUMMONING',
+        '346': 'RPL_INVITELIST',
+        '347': 'RPL_ENDOFINVITELIST',
+        '348': 'RPL_EXCEPTLIST',
+        '349': 'RPL_ENDOFEXCEPTLIST',
+        '351': 'RPL_VERSION',
+        '352': 'RPL_WHOREPLY',
+        '352': 'RPL_WHOREPLY',
+        '353': 'RPL_NAMREPLY',
+        '366': 'RPL_ENDOFNAMES',
+        '364': 'RPL_LINKS',
+        '365': 'RPL_ENDOFLINKS',
+        '367': 'RPL_BANLIST',
+        '368': 'RPL_ENDOFBANLIST',
+        '371': 'RPL_INFO',
+        '374': 'RPL_ENDOFINFO',
+        '372': 'RPL_MOTD',
+        '376': 'RPL_ENDOFMOTD',
+        '381': 'RPL_YOUREOPER',
+        '382': 'RPL_REHASHING',
+        '383': 'RPL_YOURESERVICE',
+        '391': 'RPL_TIME',
+        '392': 'RPL_USERSSTART',
+        '393': 'RPL_USERS',
+        '394': 'RPL_ENDOFUSERS',
+        '395': 'RPL_NOUSERS',
+        '200': 'RPL_TRACELINK',
+        '201': 'RPL_TRACECONNECTING',
+        '202': 'RPL_TRACEHANDSHAKE',
+        '203': 'RPL_TRACEUNKNOWN',
+        '204': 'RPL_TRACEOPERATOR',
+        '205': 'RPL_TRACEUSER',
+        '206': 'RPL_TRACESERVER',
+        '207': 'RPL_TRACESERVICE',
+        '208': 'RPL_TRACENEWTYPE',
+        '209': 'RPL_TRACECLASS',
+        '210': 'RPL_TRACERECONNECT',
+        '261': 'RPL_TRACELOG',
+        '262': 'RPL_TRACEEND',
+        '211': 'RPL_STATSLINKINFO',
+        '212': 'RPL_STATSCOMMANDS',
+        '219': 'RPL_ENDOFSTATS',
+        '242': 'RPL_STATSUPTIME',
+        '243': 'RPL_STATSOLINE',
+        '221': 'RPL_UMODEIS',
+        '234': 'RPL_SERVLIST',
+        '235': 'RPL_SERVLISTEND',
+        '251': 'RPL_LUSERCLIENT',
+        '252': 'RPL_LUSEROP',
+        '253': 'RPL_LUSERUNKNOWN',
+        '254': 'RPL_LUSERCHANNELS',
+        '255': 'RPL_LUSERME',
+        '256': 'RPL_ADMINME',
+        '257': 'RPL_ADMINLOC1',
+        '258': 'RPL_ADMINLOC2',
+        '259': 'RPL_ADMINEMAIL',
+        '263': 'RPL_TRYAGAIN',
+
+        # Error
+        '401': 'ERR_NOSUCHNICK',
+        '402': 'ERR_NOSUCHSERVER',
+        '403': 'ERR_NOSUCHCHANNEL',
+        '404': 'ERR_CANNOTSENDTOCHAN',
+        '405': 'ERR_TOOMANYCHANNELS',
+        '406': 'ERR_WASNOSUCHNICK',
+        '407': 'ERR_TOOMANYTARGETS',
+        '408': 'ERR_NOSUCHSERVICE',
+        '409': 'ERR_NOORIGIN',
+        '411': 'ERR_NORECIPIENT',
+        '412': 'ERR_NOTEXTTOSEND',
+        '413': 'ERR_NOTOPLEVEL',
+        '414': 'ERR_WILDTOPLEVEL',
+        '415': 'ERR_BADMASK',
+        '421': 'ERR_UNKNOWNCOMMAND',
+        '422': 'ERR_NOMOTD',
+        '423': 'ERR_NOADMININFO',
+        '424': 'ERR_FILEERROR',
+        '431': 'ERR_NONICKNAMEGIVEN',
+        '432': 'ERR_ERRONEUSNICKNAME',
+        '433': 'ERR_NICKNAMEINUSE',
+        '436': 'ERR_NICKCOLLISION',
+        '437': 'ERR_UNAVAILRESOURCE',
+        '441': 'ERR_USERNOTINCHANNEL',
+        '442': 'ERR_NOTONCHANNEL',
+        '443': 'ERR_USERONCHANNEL',
+        '444': 'ERR_NOLOGIN',
+        '445': 'ERR_SUMMONDISABLED',
+        '446': 'ERR_USERSDISABLED',
+        '451': 'ERR_NOTREGISTERED',
+        '461': 'ERR_NEEDMOREPARAMS',
+        '462': 'ERR_ALREADYREGISTRED',
+        '463': 'ERR_NOPERMFORHOST',
+        '464': 'ERR_PASSWDMISMATCH',
+        '465': 'ERR_YOUREBANNEDCREEP',
+        '466': 'ERR_YOUWILLBEBANNED',
+        '467': 'ERR_KEYSET',
+        '471': 'ERR_CHANNELISFULL',
+        '472': 'ERR_UNKNOWNMODE',
+        '473': 'ERR_INVITEONLYCHAN',
+        '474': 'ERR_BANNEDFROMCHAN',
+        '475': 'ERR_BADCHANNELKEY',
+        '476': 'ERR_BADCHANMASK',
+        '477': 'ERR_NOCHANMODES',
+        '478': 'ERR_BANLISTFULL',
+        '481': 'ERR_NOPRIVILEGES',
+        '482': 'ERR_CHANOPRIVSNEEDED',
+        '483': 'ERR_CANTKILLSERVER',
+        '484': 'ERR_RESTRICTED',
+        '485': 'ERR_UNIQOPPRIVSNEEDED',
+        '491': 'ERR_NOOPERHOST',
+        '501': 'ERR_UMODEUNKNOWNFLAG',
+        '502': 'ERR_USERSDONTMATCH',
+
+        # Reserved
+        '231': 'RPL_SERVICEINFO',
+        '232': 'RPL_ENDOFSERVICES',
+        '233': 'RPL_SERVICE',
+        '300': 'RPL_NONE',
+        '316': 'RPL_WHOISCHANOP',
+        '361': 'RPL_KILLDONE',
+        '362': 'RPL_CLOSING',
+        '363': 'RPL_CLOSEEND',
+        '373': 'RPL_INFOSTART',
+        '384': 'RPL_MYPORTIS',
+        '213': 'RPL_STATSCLINE',
+        '214': 'RPL_STATSNLINE',
+        '215': 'RPL_STATSILINE',
+        '216': 'RPL_STATSKLINE',
+        '217': 'RPL_STATSQLINE',
+        '218': 'RPL_STATSYLINE',
+        '240': 'RPL_STATSVLINE',
+        '241': 'RPL_STATSLLINE',
+        '244': 'RPL_STATSHLINE',
+        '244': 'RPL_STATSSLINE',
+        '246': 'RPL_STATSPING',
+        '247': 'RPL_STATSBLINE',
+        '250': 'RPL_STATSDLINE',
+        '492': 'ERR_NOSERVICEHOST',
+
+    # IRC v3.1
+        # SASL
+        '900': 'RPL_LOGGEDIN',
+        '901': 'RPL_LOGGEDOUT',
+        '902': 'ERR_NICKLOCKED',
+        '903': 'RPL_SASLSUCCESS',
+        '904': 'ERR_SASLFAIL',
+        '905': 'ERR_SASLTOOLONG',
+        '906': 'ERR_SASLABORTED',
+        '907': 'ERR_SASLALREADY',
+        '908': 'RPL_SASLMECHS',
+
+    # IRC v3.2
+        # Metadata
+        '760': 'RPL_WHOISKEYVALUE',
+        '761': 'RPL_KEYVALUE',
+        '762': 'RPL_METADATAEND',
+        '764': 'ERR_METADATALIMIT',
+        '765': 'ERR_TARGETINVALID',
+        '766': 'ERR_NOMATCHINGKEY',
+        '767': 'ERR_KEYINVALID',
+        '768': 'ERR_KEYNOTSET',
+        '769': 'ERR_KEYNOPERMISSION',
+
+        # Monitor
+        '730': 'RPL_MONONLINE',
+        '731': 'RPL_MONOFFLINE',
+        '732': 'RPL_MONLIST',
+        '733': 'RPL_ENDOFMONLIST',
+        '734': 'ERR_MONLISTFULL',
+}
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod(sys.modules['__main__'])
