@@ -38,6 +38,7 @@ import textwrap
 import traceback
 
 from . import ansi, conf, ircutils, registry, utils
+from .utils import minisix
 
 deadlyExceptions = [KeyboardInterrupt, SystemExit]
 
@@ -95,7 +96,7 @@ class StdoutStreamHandler(logging.StreamHandler):
         if record.levelname != 'ERROR' and conf.supybot.log.stdout.wrap():
             # We check for ERROR there because otherwise, tracebacks (which are
             # already wrapped by Python itself) wrap oddly.
-            if not isinstance(record.levelname, basestring):
+            if not isinstance(record.levelname, minisix.string_types):
                 print(record)
                 print(record.levelname)
                 print(utils.stackTrace())
@@ -108,7 +109,7 @@ class StdoutStreamHandler(logging.StreamHandler):
         if conf.supybot.log.stdout() and not conf.daemonized:
             try:
                 logging.StreamHandler.emit(self, record)
-            except ValueError as e: # Raised if sys.stdout is closed.
+            except ValueError: # Raised if sys.stdout is closed.
                 self.disable()
                 error('Error logging to stdout.  Removing stdout handler.')
                 exception('Uncaught exception in StdoutStreamHandler:')
@@ -126,16 +127,17 @@ class StdoutStreamHandler(logging.StreamHandler):
 class BetterFileHandler(logging.FileHandler):
     def emit(self, record):
         msg = self.format(record)
-        if not hasattr(types, "UnicodeType"): #if no unicode support...
+        try:
             self.stream.write(msg)
-            self.stream.write(os.linesep)
-        else:
+        except (UnicodeError, TypeError):
             try:
-                self.stream.write(msg)
-                self.stream.write(os.linesep)
-            except UnicodeError:
                 self.stream.write(msg.encode("utf8"))
-                self.stream.write(os.linesep)
+            except (UnicodeError, TypeError):
+                try:
+                    self.stream.write(msg.encode("utf8").decode('ascii', 'replace'))
+                except (UnicodeError, TypeError):
+                    self.stream.write(repr(msg))
+        self.stream.write(os.linesep)
         try:
             self.flush()
         except OSError as e:
@@ -182,16 +184,16 @@ conf.registerGlobalValue(conf.supybot.directories, 'log',
 
 _logDir = conf.supybot.directories.log()
 if not os.path.exists(_logDir):
-    os.mkdir(_logDir, 0755)
+    os.mkdir(_logDir, 0o755)
 
 pluginLogDir = os.path.join(_logDir, 'plugins')
 
 if not os.path.exists(pluginLogDir):
-    os.mkdir(pluginLogDir, 0755)
+    os.mkdir(pluginLogDir, 0o755)
 
 try:
     messagesLogFilename = os.path.join(_logDir, 'messages.log')
-    _handler = BetterFileHandler(messagesLogFilename)
+    _handler = BetterFileHandler(messagesLogFilename, encoding='utf8')
 except EnvironmentError as e:
     raise SystemExit('Error opening messages logfile (%s).  ' \
           'Generally, this is because you are running Supybot in a directory ' \
@@ -359,14 +361,14 @@ def firewall(f, errorHandler=None):
     def m(self, *args, **kwargs):
         try:
             return f(self, *args, **kwargs)
-        except Exception as e:
+        except Exception:
             if testing:
                 raise
             logException(self)
             if errorHandler is not None:
                 try:
                     return errorHandler(self, *args, **kwargs)
-                except Exception as e:
+                except Exception:
                     logException(self, 'Uncaught exception in errorHandler')
     m = utils.python.changeFunctionName(m, f.__name__, f.__doc__)
     return m
@@ -378,7 +380,7 @@ class MetaFirewall(type):
             if hasattr(base, '__firewalled__'):
                 cls.updateFirewalled(firewalled, base.__firewalled__)
         cls.updateFirewalled(firewalled, classdict.get('__firewalled__', []))
-        for (attr, errorHandler) in firewalled.iteritems():
+        for (attr, errorHandler) in firewalled.items():
             if attr in classdict:
                 classdict[attr] = firewall(classdict[attr], errorHandler)
         return super(MetaFirewall, cls).__new__(cls, name, bases, classdict)
@@ -394,6 +396,7 @@ class MetaFirewall(type):
         for attr in __firewalled__:
             firewalled[attr] = cls.getErrorHandler(__firewalled__, attr)
     updateFirewalled = classmethod(updateFirewalled)
+Firewalled = MetaFirewall('Firewalled', (), {})
 
 
 class PluginLogFilter(logging.Filter):
