@@ -30,7 +30,6 @@
 
 import re
 import os
-import sys
 import time
 import json
 import codecs
@@ -38,6 +37,7 @@ import string
 import textwrap
 
 from . import utils, i18n
+from .utils import minisix
 _ = i18n.PluginInternationalization()
 
 def error(s):
@@ -66,7 +66,7 @@ class NonExistentRegistryEntry(RegistryException, AttributeError):
     # raise an AttributeError if a registry entry does not exist.
     pass
 
-ENCODING = 'string_escape' if sys.version_info[0] < 3 else 'unicode_escape'
+ENCODING = 'string_escape' if minisix.PY2 else 'unicode_escape'
 decoder = codecs.getdecoder(ENCODING)
 encoder = codecs.getencoder(ENCODING)
 
@@ -108,9 +108,24 @@ def open_registry(filename, clear=False):
     _lastModified = time.time()
     _fd.close()
 
+CONF_FILE_HEADER = """
+######
+# Althrough it is technically possible to do so, we do not recommend that
+# you edit this file with a text editor.
+# Whenever possible, do it on IRC using the Config plugin, which
+# checks values you set are valid before writing them to the
+# configuration.
+# Moreover, if you edit this file while the bot is running, your
+# changes may be lost.
+######
+
+
+"""
+
 def close(registry, filename, private=True):
     first = True
     fd = utils.file.AtomicFile(filename)
+    fd.write(CONF_FILE_HEADER)
     for (name, value) in registry.getValues(getChildren=True):
         help = value.help()
         if help:
@@ -132,7 +147,7 @@ def close(registry, filename, private=True):
                                   value._name)
                     try:
                         lines.append('# Default value: %s\n' % x)
-                    except Exception as e:
+                    except Exception:
                         exception('Exception printing default value of %s:' %
                                   value._name)
             lines.append('###\n')
@@ -177,7 +192,7 @@ def join(names):
 class Group(object):
     """A group; it doesn't hold a value unless handled by a subclass."""
     def __init__(self, help='', supplyDefault=False,
-                 orderAlphabetically=False, private=False):
+                 orderAlphabetically=True, private=False):
         self._help = utils.str.normalizeWhitespace(help)
         self._name = 'unset'
         self._added = []
@@ -240,7 +255,7 @@ class Group(object):
             #print '***>', _cache[name]
             self.set(_cache[name])
         if self._supplyDefault:
-            for (k, v) in _cache.iteritems():
+            for (k, v) in _cache.items():
                 if k.startswith(self._name):
                     rest = k[len(self._name)+1:] # +1 is for .
                     parts = split(rest)
@@ -367,7 +382,7 @@ class Value(Group):
         self._lastModified = time.time()
         self.value = v
         if self._supplyDefault:
-            for (name, v) in self._children.items():
+            for (name, v) in list(self._children.items()):
                 if v.__class__ is self.X:
                     self.unregister(name)
         # We call the callback once everything is clean
@@ -502,7 +517,7 @@ class String(Value):
             v = repr(v)
         try:
             v = utils.safeEval(v)
-            if not isinstance(v, basestring):
+            if not isinstance(v, minisix.string_types):
                 raise ValueError
             self.setValue(v)
         except ValueError: # This catches utils.safeEval(s) errors too.
